# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

import os
import utils
import shutil
from poaupdater import uLogging

PROGRAM_VERSION = '1.4'
PROGRAM_COMMAND_CHECK = '--check-possible-conflicts'
PROGRAM_COMMAND_PREPARE = '--prepare-templates'
PROGRAM_COMMAND_APPLY = '--apply-templates'
PROGRAM_COMMAND_DELETE_CACHE = '--delete-cache'

KB_URL = 'https://kb.cloudblue.com/en/130479'

COMPANY_NAME = 'Ingram Micro'
BILLING_PRODUCT_NAME = 'Business Automation'
ONLINE_STORE_PRODUCT_NAME = 'Online Store'

SIMULATION_RUN_MODE = False

# Following constants should be defined with the 'prepareDirectories(programPath)' method.
PROGRAM_NAME = None
PROGRAM_DIR = None
DATA_DIR = None
OUTPUT_DIR = None
CHECK_DIR = None
BACKUP_DIR = None
LOG_FILE = None
LOG_DIR = None
CONSOLE_LOG_FILE = None

BACKUP_TIMESTAMP = None

BM_DB_HOST = None
BM_DB_NAME = None
BM_DB_USER = None
BM_DB_PASSWORD = None


def prepareDirectories(programPath):
    global PROGRAM_NAME
    global PROGRAM_DIR
    global DATA_DIR
    global OUTPUT_DIR
    global CHECK_DIR
    global BACKUP_DIR
    global LOG_FILE
    global LOG_DIR
    global CONSOLE_LOG_FILE

    PROGRAM_NAME = os.path.basename(os.path.realpath(programPath))
    PROGRAM_DIR = os.path.dirname(os.path.realpath(programPath))
    DATA_DIR = os.path.join(PROGRAM_DIR, 'lib', 'data')
    OUTPUT_DIR = os.path.join(PROGRAM_DIR, 'output')
    CHECK_DIR = os.path.join(OUTPUT_DIR, '.check')
    BACKUP_DIR = os.path.join(OUTPUT_DIR, '.backup', utils.timestamp())

    # Prepare constants for log files.
    LOG_DIR = os.path.join(OUTPUT_DIR, '.log')
    logFile = os.path.join(LOG_DIR, PROGRAM_NAME.replace('.py', ''))
    LOG_FILE = '%s-debug.log' % logFile
    CONSOLE_LOG_FILE = '%s-console.log' % logFile

    os.path.exists(OUTPUT_DIR) or os.mkdir(OUTPUT_DIR)

    startLogging()


def startLogging():
    global LOG_DIR
    global LOG_FILE
    global CONSOLE_LOG_FILE

    os.path.exists(LOG_DIR) or os.mkdir(LOG_DIR)

    uLogging.init2('/dev/null', False, False)

    def moveToArchive(logFile):
        if os.path.exists(logFile):
            targetLogPath = "{name}-{timestamp}.tld".format(
                name=logFile.rstrip('.log'),
                timestamp=uLogging.timestamp().replace(':', '-').replace(' ', '_'))
            shutil.move(logFile, targetLogPath)
            utils.makeArchiveByFilePath(targetLogPath)
            os.unlink(targetLogPath)

    # Rotate console log if exists
    moveToArchive(CONSOLE_LOG_FILE)

    # Rotate debug log if exists
    moveToArchive(LOG_FILE)

    uLogging.init2(LOG_FILE, True, False)
    uLogging.do_time = False
