# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

import constants
import utils
import os
import re

from version import VersionManager


class HashedTemplatesStorageException(Exception):
    pass


class HashedTemplatesStorage:
    def __init__(self, replaceCurrentVersionBy=None):
        self.storageDir = None
        self.versionManager = VersionManager(replaceCurrentVersionBy)

        self.hashFileNamePattern = '{file_name}.{digest}'

    def init(self):
        currentVersion = self.versionManager.getCurrentVersion().buildVersion
        self.storageDir = os.path.join(constants.DATA_DIR, '.hash-%s' % currentVersion)
        os.path.exists(self.storageDir) or os.mkdir(self.storageDir)

    def putResolvedTemplate(self, sourceTemplateDigest, resolvedTemplatePath):
        """
        Put a resolve variant of template to hash storage.
        :param sourceTemplateDigest: MD5 hash digest of original template  
        :param resolvedTemplatePath: 
        :return: Returns path to hashed file
        """
        fileName = os.path.basename(resolvedTemplatePath)
        destinationFilePath = self.generateFilePath(sourceTemplateDigest, templateFileName=fileName)
        if os.path.exists(destinationFilePath):
            return destinationFilePath

        utils.execCommand('cp {source} {destination} > /dev/null'.format(
            source=resolvedTemplatePath,
            destination=destinationFilePath
        ))

        return destinationFilePath

    def getResolvedTemplatePath(self, sourceTemplateDigest, templateId):
        destinationFilePath = self.generateFilePath(sourceTemplateDigest, templateId=templateId)

        return destinationFilePath if os.path.exists(destinationFilePath) else None

    def generateFilePath(self, digest, templateId=None, templateFileName=None):
        if not templateId and not templateFileName:
            raise HashedTemplatesStorageException('One of the required arguments is absent.')

        if not templateFileName:
            templateFileName = '%s.tpl' % templateId

        return os.path.join(self.storageDir, self.hashFileNamePattern.format(
            file_name=templateFileName,
            digest=digest
        ))

    @staticmethod
    def getFileDigest(filePath):
        return re.split('\s+', utils.readCommand('md5sum %s' % filePath))[0]

    def clearCache(self):
        utils.deleteDirectory(self.storageDir)
