# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

from poaupdater import uLogging
from textwrap import TextWrapper
import utils
import constants
import os
import re
import shutil

ANSI_CODE_BEGIN = '\033['
ANSI_CODE_END = 'm'

# Foreground colors
ANSI_COLOR_FG_BLACK = 30
ANSI_COLOR_FG_RED = 31
ANSI_COLOR_FG_GREEN = 32
ANSI_COLOR_FG_YELLOW = 33
ANSI_COLOR_FG_BLUE = 34
ANSI_COLOR_FG_MAGENTA = 35
ANSI_COLOR_FG_CYAN = 36
ANSI_COLOR_FG_WHITE = 37
ANSI_COLOR_FG_DEFAULT = 39

# Background colors
ANSI_COLOR_BG_BLACK = 40
ANSI_COLOR_BG_RED = 41
ANSI_COLOR_BG_GREEN = 42
ANSI_COLOR_BG_YELLOW = 43
ANSI_COLOR_BG_BLUE = 44
ANSI_COLOR_BG_MAGENTA = 45
ANSI_COLOR_BG_CYAN = 46
ANSI_COLOR_BG_WHITE = 47
ANSI_COLOR_BG_DEFAULT = 49

# Effects
ANSI_EFFECT_RESET = 0
ANSI_EFFECT_BOLD = 1
ANSI_EFFECT_FAINT = 2
ANSI_EFFECT_ITALIC = 3
ANSI_EFFECT_UNDERLINE = 4
ANSI_EFFECT_BLINK_SLOW = 5
ANSI_EFFECT_BLINK = 6
ANSI_EFFECT_NEGATIVE = 7
ANSI_EFFECT_CANCEAL = 8
ANSI_EFFECT_NORMAL = 22

isANSIDisabled = False

__output = None


def __getOutput():
    global __output
    if not __output:
        __output = Output()
    return __output


def ansi(message, ANSIOptions=None):
    return __getOutput().ansi(message, ANSIOptions)


def bold(message):
    return __getOutput().bold(message)


def warn(message):
    return __getOutput().warn(message)


def echo(message=None):
    __getOutput().echo(message)


def indent(width=None):
    if width is None:
        return __getOutput().indentWidth
    __getOutput().indent(width)


def disable_ansi():
    global isANSIDisabled
    isANSIDisabled = True


def get_columns():
    return __getOutput().width


class Output:
    def __init__(self):
        self.indentWidth = 0
        self.width = int(utils.readCommand('stty size').split()[1])
        self.consoleLogFile = file(constants.CONSOLE_LOG_FILE, 'a')

    def indent(self, width=None):
        self.indentWidth = int(width or 0)

    def echo(self, message=None):
        if message is None:
            message = ''

        indentLine = '{line:<{width}}'.format(line='', width=self.indentWidth)
        wrapper = TextWrapper(
            initial_indent=indentLine,
            subsequent_indent=indentLine,
            replace_whitespace=True,
            drop_whitespace=True,
            width=self.width,
        )

        def wrapLine(text, endWithLineBrake=False):
            return u'{line}{line_break}'.format(
                line='\n'.join(wrapper.wrap(text)),
                line_break='\n' if endWithLineBrake else '')

        for line in message.split('\n'):
            # Write line to console output
            wrapper.width = self.width
            uLogging.log(wrapLine(line))

            # Write output to log file
            wrapper.width = 1000  # Don't wrap, indent only
            self.consoleLogFile.write(re.sub(r'\033\[\d+m', '', wrapLine(line, True)).encode('utf-8'))

    def ansi(self, message, ANSIOptions=None):
        global isANSIDisabled
        if isANSIDisabled:
            return message

        if not ANSIOptions:
            ANSIOptions = []

        ANSICommands = ''.join([self.getANSICommand(code) for code in ANSIOptions])
        return '%s%s%s' % (ANSICommands, message, self.getANSICommand(ANSI_EFFECT_RESET))

    def bold(self, message):
        return self.ansi(message, [ANSI_EFFECT_BOLD])

    def warn(self, message):
        return self.ansi(message, [ANSI_COLOR_FG_RED, ANSI_EFFECT_BOLD])

    @staticmethod
    def getANSICommand(code):
        return '%s%d%s' % (ANSI_CODE_BEGIN, code, ANSI_CODE_END)
