# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

from poaupdater import uPEM, uBilling, uSysDB, uConfig, PEMVersion
from sys import path

import constants

dbConnect = None
storeTemplatesDates = {}


class PbaDbConnectionException(Exception):
    pass


class DBConfig:
    def __init__(self, dbhost, dbname, dbuser, dbpassword, dbtype, odbc_driver=None):
        self.database_host = dbhost
        self.database_port = ''
        self.database_name = dbname
        self.dsn_login = dbuser
        self.dsn_passwd = dbpassword
        self.database_type = dbtype
        self.database_odbc_driver = odbc_driver
        self.reinstall = False


def initPBA():
    con = uSysDB.connect()

    try:
        version = PEMVersion.getPEMVersion().version
        if (version[0] > 8 or (version[0] == 8 and version[1] >= 3)):
            uSysDB.init(DBConfig(
                constants.BM_DB_HOST or uBilling.PBAConf.getBillingDBHost(),
                constants.BM_DB_NAME or uBilling.PBAConf.getBillingDBName(),
                constants.BM_DB_USER or uBilling.PBAConf.getBillingDBUser(),
                constants.BM_DB_PASSWORD or uBilling.PBAConf.getBillingDBProp('DB_PASSWD'),
                'PGSQL'
            ))
        else:      
            platform, paPath = uPEM.getHostInfo(con, 1)
            path.append('%s/%s' % (paPath, 'roles/PBACommon'))
            import PBARoleHelper
            uSysDB.init(DBConfig(
                constants.BM_DB_HOST or PBARoleHelper.getGlobalConfValue('pba.app.host.node.id', 'DB_HOST'),
                constants.BM_DB_NAME or 'pba',
                constants.BM_DB_USER or 'pba',
                constants.BM_DB_PASSWORD or PBARoleHelper.getGlobalConfValue('pba.db.host.node.id', 'DB_PASSWD'),
                'PGSQL'
            ))

        global dbConnect
        dbConnect = uSysDB.connect()
    except BaseException as e:
        if -1 < e.message.find('password authentication failed for user'):
            raise PbaDbConnectionException()
        raise


def initPOA():
    uSysDB.init(uConfig.Config())


def getStoreTemplatesDates():
    global dbConnect
    global storeTemplatesDates

    if storeTemplatesDates:
        return storeTemplatesDates

    initPBA()
    try:
        cur = dbConnect.cursor()
        cur.execute('SELECT "StoreID", "LayoutID", "DateArc" FROM "StoreLayout" ORDER BY "StoreID" ASC')

        lastStoreID = None
        for row in cur.fetchall():
            storeId = row[0]
            templateId = row[1].decode('utf8')
            updatedDate = row[2]

            if storeId != lastStoreID:
                storeTemplatesDates[storeId] = {}
                lastStoreID = storeId

            storeTemplatesDates[storeId][templateId] = updatedDate
    finally:
        initPOA()

    return storeTemplatesDates
