# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

from codecs import open
from os import listdir, path
import tarfile
import constants


class SnapshotTemplate:
    def __init__(self,
                 templateId=None,
                 version=None,
                 priorVersion=None
                 ):

        self.templateId = templateId
        self.version = version
        self.priorVersion = priorVersion


class SnapshotManager:
    def __init__(self):
        self.snapshotDir = path.join(constants.DATA_DIR, 'templates-snapshot')
        self.sourceSnapshotFile = path.join(constants.DATA_DIR, 'templates-snapshot.tar.gz')

        if not self.isUnpacked():
            self.unpack()

    def unpack(self):
        """
        Unpack snapshot's tarball
        :return:
        """
        arc = tarfile.open(self.sourceSnapshotFile, 'r:gz')
        for tarinfo in arc:
            arc.extract(tarinfo, path=constants.DATA_DIR)
        arc.close()

    def isUnpacked(self):
        """
        :rtype bool:
        """
        return self.snapshotDir in listdir(constants.DATA_DIR)

    def getSnapshotVersion(self, BMVersion):
        """
        Get appropriated snapshot version to selected BM version
        :param BMVersion:
        :rtype bool:
        """
        snapshotVersions = self.getVersions()
        for version in snapshotVersions:
            if version == BMVersion:
                return version

            strippedVersion = self.stripVersion(BMVersion)
            if version == strippedVersion:
                return strippedVersion

        return None

    def getFirstSnapshotVersion(self):
        return self.getVersions()[0]

    def getPriorSnapshotVersion(self, BMVersion):
        versions = self.getVersions()
        priorVersion = versions[0]

        for version in versions:
            if version == BMVersion:
                return priorVersion

            strippedVersion = self.stripVersion(BMVersion)
            if version == strippedVersion:
                return priorVersion

            priorVersion = version

    def getVersionPath(self, snapshotVersion):
        return path.join(self.snapshotDir, snapshotVersion)

    def getTemplates(self, BMVersion, templateId=None):
        """
        Get templates from snapshot for selected BM version
        Retrieve all templates or one filtered by templateId
        :param BMVersion:
        :param templateId:
        :rtype: list of SnapshotTemplate
        """
        snapshotVersion = self.getSnapshotVersion(BMVersion)
        if snapshotVersion is None:
            snapshotVersion = self.getFirstSnapshotVersion()

        templates = {}

        snapshotDir = path.join(self.snapshotDir, snapshotVersion)
        templateFiles = [f for f in listdir(snapshotDir) if path.isfile(path.join(snapshotDir, f))]
        for filename in templateFiles:
            foundTemplateId = filename.replace('.tpl', '')

            if templateId and templateId != foundTemplateId:
                continue

            f = open(path.join(snapshotDir, filename), 'r', 'utf-8')
            try:
                templates[foundTemplateId] = SnapshotTemplate(
                    templateId=foundTemplateId,
                    version=snapshotVersion,
                    priorVersion=self.getPriorSnapshotVersion(BMVersion)
                )
            finally:
                f.close()

        return templates

    def getVersions(self):
        """
        Retrieve snapshot's versions
        :rtype list:
        """
        directories = listdir(self.snapshotDir)
        directories.sort()
        return directories

    @staticmethod
    def stripVersion(BMVersion):
        """
        Strip selected version
        Example: 7.0.1-11111 => 7.0.1
        :param str BMVersion:
        :rtype str:
        """
        return BMVersion.split('-')[0]
