# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

from poaupdater import uLogging
from pba import getStoreList, getStoreTemplateList, PbaException
from pbadb import PbaDbConnectionException
from output import echo
import traceback
import constants


class StoreTemplateManager:
    def __init__(self, storeId=None):
        # Active stores: [ {'store_id': <store_id>, ... }, ... ]
        self.stores = getStoreList()

        # Templates: { <store_id>: [templates] }
        self.templates = {}

        ok = True
        for sid, store in [[_sid, _store] for _sid, _store in self.stores.items() if not storeId or _sid == storeId]:
            echo('Loading data for %s...' % self.stores[sid].fullName)
            if not self.loadTemplates(sid):
                ok = False


        if not ok:
            echo('An error occurred during the loading of the data from Billing.\n'
                 'To learn more, see the log file:\n%s' % constants.LOG_FILE)

    def getTemplates(self, storeId=None):
        """
        Retrieve a dictionary with templates separated by store_id
        :param int storeId:
        :rtype: dict
        """
        if storeId and not [sid for sid, store in self.stores.items() if sid == storeId]:
            return {}

        if storeId in self.templates:
            return {storeId: self.templates[storeId]}

        return self.templates

    def loadTemplates(self, storeId):
        if storeId in self.templates.keys():
            return True

        try:
            self.templates[storeId] = getStoreTemplateList(storeId, isExtended=True)
            return True
        except PbaDbConnectionException:
            raise
        except PbaException as e:
            self.stores.pop(storeId, None)
            echo(e.message)
            echo()
            return False
        except:
            self.stores.pop(storeId, None)
            uLogging.debug(traceback.format_exc())
            echo('ERROR: Unable to retrieve data for the online store #%d. This store will be skipped.' % storeId)
            echo()
            return False

    def getStores(self):
        return self.stores
