# -*- coding: utf-8 -*-

"""
    ©2019 Ingram Micro Inc. all Rights Reserved
"""

import functools

from poaupdater import uUtil, uLogging
import constants
import os
import subprocess as sp
import time


class UtilException(Exception):
    pass


def execCommand(command, raiseError=False):
    _, _, status = __exec(command, raiseError)

    return status


def readCommand(command, raiseError=False):
    output, _, _ = __exec(command, raiseError)

    return output


def __exec(command, raiseError=False):
    if type(command) != list:
        command = ['bash', '-c', command]

    uLogging.debug("Executing the command: '%s'", command)

    cmd = sp.Popen(command, close_fds=True, stdout=sp.PIPE, stderr=sp.PIPE)

    output, error = cmd.communicate()
    status = cmd.returncode

    uLogging.debug('Output:')
    uLogging.debug(output)
    if error:
        uLogging.debug('Error:')
        uLogging.debug(error.strip())

    if raiseError and status:
        raise UtilException(error)

    return output, error, status


def deleteDirectory(directory):
    os.path.exists(directory) and execCommand('rm -rf %s' % directory)


def copyFile(sourceFile, destinationFile, raiseError=False):
    if os.path.exists(destinationFile):
        os.unlink(destinationFile)

    return execCommand(
        'cp {source} {destination} > /dev/null'.format(source=sourceFile, destination=destinationFile), raiseError
    )


def timestamp():
    now = time.time()
    return '%s.%03d' % (time.strftime('%Y-%m-%d_%H-%M-%S'), int((now - int(now)) * 1000))


def makeArchiveByDirectory(directory):
    archiveName = os.path.basename(directory)
    _, error, status = __exec("cd '{path}'/.. && tar -zcf '{name}.tar.gz' '{name}' > /dev/null".format(
        path=directory, name=archiveName))

    if status:
        raise UtilException(error)


def makeArchiveByFilePath(targetPath):
    archiveName = os.path.basename(targetPath)
    archiveDir = os.path.dirname(targetPath)
    _, error, status = __exec("cd '{path}' && tar -zcf '{name}.tar.gz' '{name}' > /dev/null".format(
        path=archiveDir, name=archiveName))

    if status:
        raise UtilException(error)


def makeDirectory(directory):
    _, error, status = __exec('mkdir -p {directory} > /dev/null'.format(directory=directory))

    if status:
        raise UtilException(error)


def forbid_for_simulation_mode(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if not constants.SIMULATION_RUN_MODE:
            func(*args, **kwargs)
        else:
            uLogging.debug("The utility is in simulation mode. "
                           "The '%s' method call will be skipped." % func.__name__)

    return wrapper
