<!--
  country.tpl
-->
<script>
    require([
        "dojo/_base/array",
        "dojo/_base/lang",
        "dojo/dom",
        "dojo/on",
        "aps/Memory",
        "aps/load",
        "aps/ready!"
    ], function(array, lang, dom, on, Memory, load) {
        "use strict";

        var countries = [
            {foreach name=country_outer item=country from=$config->countryList}
            { name: "{$country.Name}", id: "{$country.Code}" }{if !$smarty.foreach.country_outer.last},{/if}

            {/foreach}
        ];

        var countryStore = new Memory({
            data: getCountryStoreData()
        });

        function getCountryString(country){
            return country.name + " (" + country.id + ")";
        }

        function getCountryByName(countryName){
            var foundCountry = null;
            array.some(countries, function(country) {
                if (countryName === getCountryString(country)) {
                    foundCountry = country;
                    return true;
                }
            });
            return foundCountry;
        }

        function getCountryById(countryId){
            var foundCountry = null;
            array.some(countries, function(country) {
                if (countryId === country.id) {
                    foundCountry = country;
                    return true;
                }
            });
            return foundCountry;
        }

        function getCountryStoreData(){
            var data = [];
            countries.sort(function (c1, c2) {
                return c1.name < c2.name ? -1 : 1;
            }).forEach(function (country) {
                data.push({ id: country.id, name: getCountryString(country) })
            });
            return data;
        }

        load(["aps/ComboBox", {
            id: "CountrySelector",
            store: countryStore,
            required: true,
            autoWidth: false,
            minLength: 0,
            value: countryStore.get(CountryID).name,
            onClick: function () {
                dijit.selectInputText(this.textbox);
                this.toggleDropDown();
            },
            onChange: function () {
                var country = getCountryByName(this.value);
                if (country != null) {
                    this.set("pattern", ".*");
                    var hiddenSelector = dom.byId("CountryID");
                    hiddenSelector.value = country.id;
                    on.emit(hiddenSelector, "change", {
                        bubbles: true,
                        cancelable: true
                    });
                } else {
                    this.set("pattern",  {literal}"[0]{0}"{/literal});
                    this.focus();
                }
            }
        }], "countrySelectorDiv").then(function () {
            var hiddenSelector = dom.byId("CountryID");
            var countrySelector = dom.byId("CountrySelector");

            on(hiddenSelector, "change", function (event) {
                // countrySelector.value ~ United States of America (us); event.target.value ~ us
                if (event && lang.exists('target.value', event) && countrySelector && lang.exists('value', countrySelector)) {
                    var countryCodePart = '(' + event.target.value + ')'; // (us)
                    if (countrySelector.value.slice(-countryCodePart.length) !== countryCodePart) {
                        countrySelector.value = getCountryString(getCountryById(event.target.value));
                    }
                }
            });
        });
    });
</script>

<table>
    <tr>
        <td class="fieldLabel">
            <label for="CountrySelector">{$smarty.session.LANG.COUNTRY}</label>
        </td>
    </tr>
    <tr>
        <td align="left">
            <div class="fieldEntry">
                <div id="countrySelectorDiv"></div>
                <select class="CountryID" name="CountryID" id="CountryID" title="{$smarty.session.LANG.COUNTRY}" onchange="verifyAddress()" style="display: none">
                    {foreach name=country_outer item=country from=$config->firstCountryList}
                        <option value="{$country.Code}"
                                TaxRegReqPers="{$country.TaxRegReqPers}"
                                TaxRegReqComp="{$country.TaxRegReqComp}"
                                StateProvinceReq="{$country.StateProvinceReq}"
                                DefaultPhoneCountryCode="{$country.DefaultPhoneCountryCode}"
                                PhoneAreaCodeReq="{$country.PhoneAreaCodeReq}"
                                zipRegex="{$country.ZipRegEx}">{$country.Name}</option>
                    {/foreach}
                    {if $config->firstCountryList}
                        <option disabled> -- </option>
                    {/if}
                    {foreach name=country_outer item=country from=$config->countryList}
                        <option value="{$country.Code}"
                                TaxRegReqPers="{$country.TaxRegReqPers}"
                                TaxRegReqComp="{$country.TaxRegReqComp}"
                                StateProvinceReq="{$country.StateProvinceReq}"
                                DefaultPhoneCountryCode="{$country.DefaultPhoneCountryCode}"
                                PhoneAreaCodeReq="{$country.PhoneAreaCodeReq}"
                                zipRegex="{$country.ZipRegEx}">{$country.Name}</option>
                    {/foreach}
                </select>
            </div>
        </td>
    </tr>

    {GetCustomTemplate name="accattr.tpl" assign="file"}
    {include file=$file filter="COUNTRY"}
</table>



<!--
  end of country.tpl
-->
