<!--
  country.tpl
-->
<script>
    aps = { context: { bs: true } };
    require([
        "dojo/_base/array",
        "dojo/_base/lang",
        "dojo/dom",
        "dijit/registry",
        "dojo/on",
        "aps/Memory",
        "aps/load",
        "aps/ready!"
    ], function(array, lang, dom, registry, on, Memory, load) {
        "use strict";

        var preferredCountriesSeparator = '--';
        var countries = [];
        var preferredCountries = [];

        {foreach name=country_outer item=country from=$config->firstCountryList}
        preferredCountries.push({
            id: '{$country.Code|escape:'javascript'}',
            name: '{$country.Name|escape:'javascript'}'
        });
        {/foreach}

        {foreach name=country_outer item=country from=$config->countryList}
        countries.push({
            id: '{$country.Code|escape:'javascript'}',
            name: '{$country.Name|escape:'javascript'}'
        });
        {/foreach}

        var countryStore = new Memory({
            data: getCountryStoreData()
        });

        function getCountryFullName(country) {
            if (country) {
                return country.name + ' (' + country.id + ')';
            }

            return '';
        }

        function getCountryByName(countryName){
            var foundCountry = null;
            array.some(countryStore.data, function(country) {
                if (countryName === country.name || countryName === country.fullName) {
                    foundCountry = country;
                    return true;
                }
            });
            return foundCountry;
        }

        function getCountryById(countryId){
            var foundCountry = null;
            array.some(countryStore.data, function(country) {
                if (countryId === country.id) {
                    foundCountry = country;
                    return true;
                }
            });
            return foundCountry;
        }

        function getCountryStoreData(){
        {literal}
            var data = [];
            if (preferredCountries.length) {
                preferredCountries.forEach(function (country) {
                    data.push({id: country.id, name: country.name, fullName: getCountryFullName(country)});
                });
                data.push({id: null, name: null, fullName: preferredCountriesSeparator});
            }

            // Add other countries sorted by alphabetical
            countries.forEach(function (country) {
                data.push({id: country.id, name: country.name, fullName: getCountryFullName(country)});
            });

            return data;
        {/literal}
        }

        load(["aps/ComboBox", {
            id: "CountrySelector",
            store: countryStore,
            required: true,
            autoWidth: false,
            minLength: 0,
            item: getCountryById(CountryID),

            lastValidItem: {literal}{}{/literal},

            optionTemplate: {literal}'${fullName}'{/literal},
            searchAttr: 'customName',

            onChange: function () {
                var country = getCountryByName(this.value);
                if (!country) {
                    country = getCountryById(this.value);
                }
                if (country && country.id) {
                    this.set('item', country);
                    var hiddenSelector = dom.byId("CountryID");
                    hiddenSelector.value = country.id;
                    on.emit(hiddenSelector, "change", {
                        bubbles: true,
                        cancelable: true
                    });
                } else if (!country.id) {
                    // Country item is unknown or is a separator
                    // Try to return a last validated item
                    this.set('item', this.get('lastValidItem'));
                }
                this.onBlur();
            },
            extraValidator: function () {
                try {
                    var valid = false;
                    if (this.get('focused')) {
                        valid = true;
                    } else {
                        var country = this.get('item');
                        if (country.name === this.get('value') && country.id) {
                            valid = true;
                        }
                    }
                    if (valid) {
                        this.set('lastValidItem', this.get('item'));
                    }
                    return valid;
                } catch (ignored) {
                    return false;
                }
            }
        }], 'countrySelectorDiv').then(function () {
            var domCountrySelector = dom.byId('CountrySelector');

            // Fire 'onchange' event for dom element with id = 'CountrySelector'
            // This trick is needed for first initialization of all related controls:
            //     Zip / Postal Code
            //     City
            //     State / Province
            on.emit(domCountrySelector, 'change', {
                bubbles: true,
                cancelable: true
            });

            // To force opening a dropdown list on each click on Country control
            domCountrySelector.onclick = function () {
                registry.byId('CountrySelector').focus();
            }
        });
    });
</script>

<table>
    <tr>
        <td class="fieldLabel">
            <label for="CountrySelector">{$smarty.session.LANG.COUNTRY}</label>
        </td>
    </tr>
    <tr>
        <td align="left">
            <div class="fieldEntry">
                <div id="countrySelectorDiv"{if $isMobile} style="display: none;"{/if}></div>
                <select class="CountryID" name="CountryID" id="CountryID" title="{$smarty.session.LANG.COUNTRY}" onchange="verifyAddress()" {if !$isMobile} style="display: none;"{/if}>
                    {foreach name=country_outer item=country from=$config->firstCountryList}
                        <option value="{$country.Code}"
                                TaxRegReqPers="{$country.TaxRegReqPers}"
                                TaxRegReqComp="{$country.TaxRegReqComp}"
                                StateProvinceReq="{$country.StateProvinceReq}"
                                DefaultPhoneCountryCode="{$country.DefaultPhoneCountryCode}"
                                PhoneAreaCodeReq="{$country.PhoneAreaCodeReq}"
                                zipRegex="{$country.ZipRegEx}">{$country.Name}</option>
                    {/foreach}
                    {if $config->firstCountryList}
                        <option disabled> -- </option>
                    {/if}
                    {foreach name=country_outer item=country from=$config->countryList}
                        <option value="{$country.Code}"
                                TaxRegReqPers="{$country.TaxRegReqPers}"
                                TaxRegReqComp="{$country.TaxRegReqComp}"
                                StateProvinceReq="{$country.StateProvinceReq}"
                                DefaultPhoneCountryCode="{$country.DefaultPhoneCountryCode}"
                                PhoneAreaCodeReq="{$country.PhoneAreaCodeReq}"
                                zipRegex="{$country.ZipRegEx}">{$country.Name}</option>
                    {/foreach}
                </select>
            </div>
        </td>
    </tr>

    {GetCustomTemplate name="accattr.tpl" assign="file"}
    {include file=$file filter="COUNTRY"}
</table>



<!--
  end of country.tpl
-->
