<!-- paci_configs_js.tpl -->
<script type="text/javascript">
	var pbaResources = {$plan.PlanRate|@json_encode}; // [LB] PBA resource rates
	var provParams = {$prov|@json_encode};
	var token = '{$token}';

	{literal}	
	var samples = [];
	var backupSchedules = [];
	var pbaResourcesById = {};
	var pbaResourcesByName = {};
	var pbaResourcesByApsType = {};
	var currentSampleIndx = 0;
	var osByApsId = {};
	
	function filterResourcesByType(resources, type){
		var result = [];
		for(idx in resources){
			var resource = resources[idx];
			if(('aps' in resource) && (resource.aps.type === type)){
				result.push(resource);
			}
		}
		return result;
	}
	var uomMultipliers = [
		{"uom": ["kb", "mb", "gb"], "multiplier": 1024},
		{"uom": ["khz", "mhz", "ghz"], "multiplier": 1000},
		{"uom": ["kbit/s", "mhz", "ghz"], "multiplier": 1000}
	]
	function convertUnits(value, fromUOM, toUOM){
		//console.log("converting " + value + " " +fromUOM+ " to " + toUOM);
		fromUOM = fromUOM.toLowerCase();
		toUOM = toUOM.toLowerCase();
        if(fromUOM === toUOM){
            return value;
        }
		for(indx in uomMultipliers){
			fromUOMIndx = uomMultipliers[indx].uom.indexOf(fromUOM);
			toUOMIndx = uomMultipliers[indx].uom.indexOf(toUOM);
            if(fromUOMIndx != -1 && toUOMIndx!=-1){
                var mult = Math.pow(uomMultipliers[indx].multiplier, (fromUOMIndx - toUOMIndx));
                var res = value*mult;
				//console.log("result: " + res);
                return res;
            }
		}
        console.error("cannot convert " + fromUOM + " to " + toUOM);
	}
	function prepareSamples(samples) {
	    //sort first by SortOrder (configured in PBA) then by price
		samples.sort(function(a,b){
			var resA = pbaResourcesById[a.resourceID];
			var resB = pbaResourcesById[b.resourceID];
			return (resA.SortOrder == resB.SortOrder) ? a.price - b.price : resA.SortOrder - resB.SortOrder;
		});
		
	    // sample widget's ID is an index in samples array
	    for (var i = 0; i < samples.length; i++) {
			var pbaRes = pbaResourcesByName["memory"];
			samples[i].hardware.memory.value = convertUnits(samples[i].hardware.memory.value, "mb", pbaRes.UnitOfMeasure);
			samples[i].hardware.memory.max = convertUnits(samples[i].hardware.memory.max, "mb", pbaRes.UnitOfMeasure);
			samples[i].hardware.memory.step = convertUnits(samples[i].hardware.memory.step, "mb", pbaRes.UnitOfMeasure);
			samples[i].hardware.memory.uom = pbaRes.UnitOfMeasure;
			
			pbaRes = pbaResourcesByName["diskspace"];
			samples[i].hardware.disk.space.value = convertUnits(samples[i].hardware.disk.space.value, "gb", pbaRes.UnitOfMeasure);
			samples[i].hardware.disk.space.max = convertUnits(samples[i].hardware.disk.space.max, "gb", pbaRes.UnitOfMeasure);
			samples[i].hardware.disk.space.value = convertUnits(samples[i].hardware.disk.space.value, "gb", pbaRes.UnitOfMeasure);
			samples[i].hardware.disk.space.uom = pbaRes.UnitOfMeasure;
			
			pbaRes = pbaResourcesByName["cpu.power"];
			samples[i].hardware.cpu.power.value = convertUnits(samples[i].hardware.cpu.power.value, "mhz", pbaRes.UnitOfMeasure);
			samples[i].hardware.cpu.power.max = convertUnits(samples[i].hardware.cpu.power.max, "mhz", pbaRes.UnitOfMeasure);
			samples[i].hardware.cpu.power.value = convertUnits(samples[i].hardware.cpu.power.value, "mhz", pbaRes.UnitOfMeasure);
			samples[i].hardware.cpu.power.uom = pbaRes.UnitOfMeasure;
			
	        samples[i].indx = i;
	    }
	}
	function loadOsTemplates(){
		var deferred = jQuery.Deferred();
		var requests = [];

		for(idx in samples){
			requests.push(jQuery.ajax({
	        url : "/aps/2/resources/" + samples[idx].aps.id + "/oses",
	        headers : {
	            'APS-Token' : token
	        }}));
		}
		jQuery.when.apply($, requests).done(function (result){
			var oses = JSON.parse(result[0]);
			for(i in oses){
				osByApsId[oses[i].aps.id] = oses[i];
			}
			deferred.resolve();
		}).fail(function(error){
			console.error("Cannot load OS templates. Make sure PACI service is running");
		});
		return deferred;
	}
	function prepareResources() {
	    var deferred = jQuery.Deferred();
	    var pbaResourcesByResourceId = {};
		var resourceIdByApsResourceId = {};

	    jQuery.map(pbaResources, function (pbaResource) {
	        pbaResourcesByResourceId[pbaResource.ResourceID] = pbaResource;
	    });
		jQuery.map(pbaResources, function (pbaResource) {
	        pbaResourcesById[pbaResource.ID] = pbaResource;
	    });
	    jQuery.ajax({
	        url : "/aps/2/resources/",
	        headers : {
	            'APS-Token' : token
	        }
	    }).done(function (rootApsResources) {
			var serviceTemplates = filterResourcesByType(rootApsResources, 'http://parallels.com/aps/types/pa/serviceTemplate/1.0');
			samples = filterResourcesByType(rootApsResources, 'http://www.parallels.com/infrastructure/virtual-environment/sample/1.0');
		    if (samples.length == 0) {
		        console.error("Samples not found in APS resources");
		        return false;
		    }
			backupSchedules = filterResourcesByType(rootApsResources, 'http://www.parallels.com/infrastructure/virtual-environment/backup-schedule/1.0');
			if(serviceTemplates.length == 0){
				console.error("Service Template not found in APS resources");
				return false;
			}
			// from "limits" we get PA resource ID's
			jQuery.ajax({
			    url : "/aps/2/resources/" + serviceTemplates[0].aps.id + "/limits",
			    headers : {
			        'APS-Token' : token
			    }
			}).done(function (limits) {
				jQuery.map(limits, function(limit){
					if('apsId' in limit){
						resourceIdByApsResourceId[limit.apsId] = limit.id;
					}
					if(limit.apsType === 'http://www.parallels.com/infrastructure/virtual-environment/context/1.0' && ('property' in limit)){
						pbaResourcesByName[limit.property] = pbaResourcesByResourceId[limit.id];
					}
					pbaResourcesByApsType[limit.apsType] = pbaResourcesByResourceId[limit.id];
				});
				for(indx in samples){
					var sample = samples[indx];
					var pbaResource = pbaResourcesByResourceId[resourceIdByApsResourceId[sample.aps.id]];
					sample.resourceID = pbaResource.ID;
					sample.price = pbaResource.RecurringFee;
				}
				prepareSamples(samples);
				loadOsTemplates().done(function(){
					deferred.resolve();
				});
			});
		});

	    return deferred;
	}
	jQuery( document ).ready(function() {
		prepareResources().done(function(){
			var pbaRes;
			jQuery('.conf_line').show();
			// samples
			jQuery('#sampleTmpl').tmpl(samples).prependTo('#topSamples');
			jQuery('.categoryPlansPanel div.planSmallPanel:nth-of-type(3n+3)').css({'margin-right':'0px'});
			//OS
			jQuery('#selectTmpl').tmpl({id: 'os', label: locale["VPSNG_RESOURCES_OS"]}).appendTo('#mainControls'); // XXXX Fix spelling: Operation - > Operating!
			// Hardware
			jQuery('#sliderTmpl').tmpl({
			    id : 'cpu_power',
				pbaRes: pbaResourcesByName['cpu.power'],
			    label : locale["VPSNG_RESOURCES_CPU"]
			}).appendTo('#hardwareControls');

			jQuery('#sliderTmpl').tmpl({
			    id : 'cpu_number',
				pbaRes : pbaResourcesByName['cpu.number'],
			    label : locale["VPSNG_RESOURCES_CPU_CORES"],
			    uom : 'Unit(s)'
			}).appendTo('#hardwareControls');

			jQuery('#sliderTmpl').tmpl({
			    id : 'memory',
				pbaRes : pbaResourcesByName['memory'],
			    label : locale["C2U_RESOURCES_RAM"]
			}).appendTo('#hardwareControls');

			jQuery('#sliderTmpl').tmpl({
			    id : 'disk_space',
				pbaRes : pbaResourcesByName['diskspace'],
			    label : locale["C2U_RESOURCES_DISK"]
			}).appendTo('#hardwareControls');
			
			//Advanced
			jQuery('#selectTmpl').tmpl({
			    id : 'backup_schedule',
			    label : locale["VPSNG_RESOURCES_BACKUP"]
			}).appendTo('#advSettingsLeft');
			
			jQuery('#selectTmpl').tmpl({
			    id : 'bandwidth',
				pbaRes : pbaResourcesByName['bandwidth'],
			    label : locale["VPSNG_RESOURCES_BANDWIDTH"]
			}).appendTo('#advSettingsRight');
			
			jQuery('#numberFieldTmpl').tmpl({
			    id : 'backup_space',
				pbaRes: pbaResourcesByName['backupSpace'],
			    label : locale["VPSNG_RESOURCES_BacupSpace"],
			    uom : 'GB'
			}).appendTo('#advSettingsLeft');
			
			jQuery('#textFieldTmpl').tmpl({
			    id : 'hostname',
			    label : locale["VPSNG_RESOURCES_HOSTNAME"],
			    hint : locale["VPSNG_RESOURCES_HOSTNAME_HINT"]
			}).appendTo('#advSettingsRight');
			
			pbaRes = pbaResourcesByName['inboundTraffic'];
			if(pbaRes != null){
				jQuery('#numberFieldTmpl').tmpl({
					id : 'traffic_in',
					label : locale["VPSNG_RESOURCES_TrafficIN"],
					pbaRes: pbaRes,
					uom : 'GB'
				}).appendTo('#advSettingsRight');
			}
			
			pbaRes = pbaResourcesByName['outboundTraffic'];
			if(pbaRes != null){
				jQuery('#numberFieldTmpl').tmpl({
					id : 'traffic_out',
					pbaRes: pbaRes,
					label : locale["VPSNG_RESOURCES_TrafficOUT"],
					uom : 'GB'
				}).appendTo('#advSettingsRight');
			}
			
			pbaRes = pbaResourcesByApsType['http://www.parallels.com/infrastructure/virtual-environment/ip-v4-address/1.0'];
			jQuery('#numberFieldTmpl').tmpl({
			    id : 'ipv4',
				pbaRes: pbaRes,
			    label : locale["VPSNG_RESOURCES_IPv4"]
			}).appendTo('#advSettingsLeft');
			
			pbaRes = pbaResourcesByApsType['http://www.parallels.com/infrastructure/virtual-environment/ip-v6-address/1.0'];
			if(pbaRes != null) {
				jQuery('#numberFieldTmpl').tmpl({
					id : 'ipv6',
					pbaRes: pbaRes,
					label : locale["VPSNG_RESOURCES_IPv6"]
				}).appendTo('#advSettingsLeft');
			}
			populateBackupSchedules();
			restoreState();
			jQuery('#loading').hide();
		});
	});
	function restoreState(){
		var sampleId = jQuery('#cloud_config_id').attr("value");
		if(sampleId == ""){
			onSampleClicked(0);
		}else{
			for(var i = 0; i < samples.length; i++){
				if(samples[i].resourceID == sampleId)
					onSampleClicked(i);
			}
		}
	}
	function populateBackupSchedules(){
		backupSchedules.sort(function (a, b) {
		    if (!a.priority )
		        a.priority = Number.MAX_VALUE;
		    if (!b.priority)
		        b.priority = Number.MAX_VALUE;
		    return a.priority - b.priority;
		});
		jQuery('#backup_schedule').html("");
		for(idx in backupSchedules){
			jQuery('#backup_schedule').append(jQuery('<option>', {value: backupSchedules[idx].aps.id, text: backupSchedules[idx].name}));
		}
	}
	function updateTotalPrice(){
		var total = Number(cloud_base_price_month) + Number(samples[currentSampleIndx].price);
		
		jQuery('[price]').map(function(){
			total += Number(jQuery(this).attr("price"));
		});
		//TODO animation like in old store (paci-controller)?
		//TODO apply red color only when some resources selected? 
		//TODO move this stuff to cloudUpdatePrice()
		if(total > 0)
			jQuery('#cloud_total_month').addClass('up');
		else
			jQuery('#cloud_total_month').removeClass('up');
		cloudUpdatePrice('cloud_total_month', total);
	}
	function onSampleClicked(indx){
		console.log('onSampleClicked(' + indx + ')');
		jQuery('#sample_'+currentSampleIndx).removeClass('vpsSelect');
		jQuery('#sample_'+indx).addClass('vpsSelect');
		currentSampleIndx = indx;
		jQuery('#cloud_config_id').val(samples[currentSampleIndx].resourceID); //used by server to preserve state
		reconfigureResourceWidgets();
		populateOsList();
		populateBandwidth();
		updateTotalPrice();
	}
	var bwOptions = [
			jQuery('<option>', {value: 5, text: '5 Mbit/sec'}),
			jQuery('<option>', {value: 10, text: '10 Mbit/sec'}),
			jQuery('<option>', {value: 50, text: '50 Mbit/sec'}),
			jQuery('<option>', {value: 100, text: '100 Mbit/sec'}),
			jQuery('<option>', {value: 500, text: '500 Mbit/sec'}),
			jQuery('<option>', {value: 1000, text: '1 Gbit/sec'})
		];
	function populateBandwidth(){
		jQuery('#bandwidth').html("");
		var pbaRes = pbaResourcesByName["bandwidth"];
		var sampleLimits = samples[currentSampleIndx].hardware.bandwidth;
		var limits = calcLimits(sampleLimits, pbaRes);
		jQuery.map(bwOptions, function(option){
			if(limits.min <= Number(option[0].value) && Number(option[0].value) <= limits.max){
				jQuery('#bandwidth').append(option);
			}
		});
		onParamChanged(null, 'bandwidth');
	}
	function calcLimits(sampleLimits, pbaRes){
		return {
			min : (pbaRes!=null && pbaRes.IncludedValue!=-1) ? Math.max(sampleLimits.value, pbaRes.IncludedValue) : sampleLimits.value,
			max : (pbaRes!=null && pbaRes.UpperLimit!=-1) ? Math.min(sampleLimits.max, pbaRes.UpperLimit) : sampleLimits.max
		}
	}
	function populateOsList(){
		jQuery('#os').html("");
		var osIds = samples[currentSampleIndx].templates.os;
		for(idx in osIds){
			jQuery('#selectOptionTmpl').tmpl({value: osIds[idx], label: osByApsId[osIds[idx]].name}).appendTo('#os');
		}
		jQuery('#os').val(samples[currentSampleIndx].templates.defaultOS);
	}
	function setSliderLimits(sliderId, sampleLimits, pbaRes){
		var slider = getSliderById(sliderId);
		var limits = calcLimits(sampleLimits, pbaRes);
		slider.n_step = sampleLimits.step;
		slider.f_setMin(limits.min);
		slider.f_setMax(limits.max);
		slider.f_setValue(limits.min);
		jQuery('#'+sliderId+'_min').text(limits.min);
		jQuery('#'+sliderId+'_max').text(limits.max);
	}
	function reconfigureResourceWidgets(){
		var sample = samples[currentSampleIndx];
		setSliderLimits('cpu_power', sample.hardware.cpu.power, pbaResourcesByName["cpu.power"]);
		setSliderLimits('cpu_number', sample.hardware.cpu.number, pbaResourcesByName["cpu.number"]);
		setSliderLimits('memory', sample.hardware.memory, pbaResourcesByName["memory"]);
		setSliderLimits('disk_space', sample.hardware.disk.space, pbaResourcesByName["diskspace"]);
	}
	function onParamChanged(e, name){
		console.log('onParamChanged(' + name + ')');
		var amount = jQuery('#' + name).val();
		var fee = jQuery('#' + name).attr('fee');
		if(fee) {
			cloudUpdatePrice('price_'+name, amount*fee);
			updateTotalPrice();
		}
	}
	function switchAdvanced(){
		jQuery('#advanced_opener').toggle();
		jQuery('#advanced').toggle();
	}
	function onSubmitForm(){
		prepareVeConfig();
		//CPU power - special case
		var cpuPower = jQuery('#slider_input_cpu_power');
		var cpuNumber = jQuery('#slider_input_cpu_number');
		cpuPower.val(cpuPower.val()*cpuNumber.val());
	}
	function prepareVeConfig() {
	    console.log('prepareVeConfig()');
	    var ve = {
	        "name" : jQuery('#hostname').val(),
	        "hostname" : jQuery('#hostname').val(),
	        "osId" : jQuery('#os').val(),
	        "sampleId" : samples[currentSampleIndx].aps.id,
	        "backupScheduleId" : jQuery('#backup_schedule').val(),
	        "hardware" : {
	            "bandwidth" : jQuery('#bandwidth').val(),
	            "cpu" : {
	                "number" : jQuery('#slider_input_cpu_number').val(),
	                "power" : convertUnits(jQuery('#slider_input_cpu_power').val(), pbaResourcesByName["cpu.power"].UnitOfMeasure, "mhz")
	            },
	            "diskspace" : convertUnits(jQuery('#slider_input_disk_space').val(), pbaResourcesByName["diskspace"].UnitOfMeasure, "gb"),
	            "memory" : convertUnits(jQuery('#slider_input_memory').val(), pbaResourcesByName["memory"].UnitOfMeasure, "mb")
	        },
	        "ipv4" : jQuery('#ipv4').val(),
	        "ipv6" : jQuery('#ipv6').val()
	    };

	    var provisioningParams = {
	        "aps" : {
	            "type" : "http://www.parallels.com/infrastructure/virtual-environment/context/1.0"
	        },
	        "environmentTemplate" : ve
	    };
	    jQuery('#aps_json').val(JSON.stringify(provisioningParams));
	}	
	{/literal}
</script>
<!-- end of paci_configs_js.tpl -->

